#!/usr/bin/env python

"""
Conversion functions and data used by XML-RPC and SOAP.

Copyright (C) 2007 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option) any
later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import datetime

# Utility classes.

class ParameterName(object):

    "A method parameter name."

    def __init__(self, ns, name):
        self.ns = ns
        self.name = name

    def __eq__(self, other):
        return self.ns, to_localName(self.name) == other.ns, to_localName(other.name)

    def __hash__(self):
        return hash(self.ns + to_localName(self.name))

    def __repr__(self):
        return "ParameterName(%s, %s)" % (repr(self.ns), repr(self.name or None))

class ParameterValue(object):

    "A method parameter value."

    def __init__(self, name, value):
        self.name = name
        self.value = value

    def convert(self, converters=None):
        conv = default_converters
        conv.update(converters or {})
        typename = self.name.ns
        localName = to_localName(self.name.name)
        if isinstance(self.value, list):
            return [item.convert(converters) for item in self.value]
        else:
            functions = conv.get(typename, {})
            function = functions.get(localName) or functions.get(None, unicode)
            return function(self.value)

    def _data(self):
        return self.convert()

    data = property(_data)

    def __eq__(self, other):
        return self.name == other.name and self.value == other.value

    def __hash__(self):
        return hash(self.value)

    def __repr__(self):
        return "ParameterValue(%s, %s)" % (repr(self.name), repr(self.value))

    # Sequence emulation.

    def __len__(self):
        return len(self.value)

    def __getitem__(self, i):
        return self.value[i]

def to_localName(name):
    return (name or "").split(":")[-1] or None

# Utility functions.

def boolean(s):
    if s.lower() == "true":
        return True
    elif s.lower() == "false":
        return False
    else:
        raise ValueError, "String value %s not convertable to boolean." % repr(s)

def iso8601(s):
    # NOTE: To be written.
    return s

default_converters = {
    "string" : {None : unicode},
    "int" : {None : int},
    "i4" : {None : int},
    "double" : {None : float},
    "boolean" : {None : boolean},
    "dateTime.iso8601" : {None : iso8601},
    "base64" : {None : str}
    }

# vim: tabstop=4 expandtab shiftwidth=4
